//
//  PAData.h
//  MoveCoachMobile
//
//  Created by Alex Morgan on 8/15/13.
//

#import <Foundation/Foundation.h>
#import "Utilities.h"

/**
 *  This Class is part of the Data Model and represents a single instance of a user physical activity entry.  The UserData Class maintains an array of PAData objects to provide a history of user physical activity.
 */

@interface PAData : NSObject<NSCoding>

/**
 *  This property represents the name of the physical activity entered into the system
 */
@property (nonatomic, strong)NSString *paName;

/**
 *  This property represents the database code associated with the physical activity entered into the system
 */
@property (nonatomic, strong)NSString *paCode;

/**
 *  This property represents the amount of time in minutes spent on the physical activity entered into the system
 */
@property (nonatomic, strong)NSString *timeMin;

/**
 *  This property represents the database category (Aerobic, Strength or Flexibility) of the physical activity entered into the system
 */
@property (nonatomic, strong)NSString *paCategory;

/**
 *  This property represents the date the physical activity was entered into the system as a string.
 *
 *  NOTE: use referenceDate for all computational, db, or locale-sensitive functions.
 */
@property (nonatomic, strong)NSString *dateCreated;

/**
 *  This property represents the date and time the physical activity was entered into the system.
 *
 *  NOTE: use this proprerty for all computational, db, or locale-sensitive functions.
 */
@property (nonatomic, strong)NSDate *referenceDate;

/**
 *  This property represents the number of calories burned for the physical activity entered into the system
 */
@property (nonatomic)float caloriesBurned;

/**
 *  This property represents the MET Value for the physical activity entered into the system
 */
@property (nonatomic)float METValue;

/**
 *  This property represents the intensity level for the physical activity entered into the system
 */
@property (nonatomic,strong)NSString *intensityLevel;

/**
 *  This method allows a PAData object to be created with a specific date.
 *
 *  @param paName - name of the activity entered
 *
 *  @param paCode - database code of the activity entered
 *
 *  @param timeMin - time spent on the activity entered
 *
 *  @param caloriesBurned - calories expended on the activity entered
 *
 *  @dateCreated - a specific NSString "date" in 'MM/dd/yy' format to set dateCreated to.
 *
 *  @param category - physical activity category the activity entered is related to
 */
- (id)initWithPAName:(NSString *)paName paCode:(NSString *)paCode timeMin:(NSString *)timeMin caloriesBurned:(float)caloriesBurned dateCreated:(NSString*)dateCreated category:(NSString *)category;

/**
 *  This method allows a PAData object to be created with a specific name, activity code, duration, calories burned, date, and category (e.g. aerobic, strength, or flexibility).
 *
 *  @param paName - name of the activity entered
 *
 *  @param paCode - database code of the activity entered
 *
 *  @param timeMin - time spent on the activity entered
 *
 *  @param caloriesBurned - calories expended on the activity entered
 *
 *  @dateCreated - a specific NSString "date" in 'MM/dd/yy' format to set dateCreated to.
 *
 *  @referenceDate - [optional] a specific NSDate to set referenceDate to if not passed, the date/time of instantiation will be used.
 *
 *  @param category - physical activity category the activity entered is related to
 */
- (id)initWithPAName:(NSString *)paName paCode:(NSString *)paCode timeMin:(NSString *)timeMin caloriesBurned:(float)caloriesBurned dateCreated:(NSString*)dateCreated referenceDate:(NSDate*)refDate category:(NSString *)category;

/**
 * This convenience method returns calories consumed per hour when performing this activity
 */
- (float)caloriesPerHour;
@end
