//
//  Utilities.h
//  MoveCoachMobile
//
//  Created by Stephen Barrs on 12/19/12.
//

#import <Foundation/Foundation.h>

//define all constant definitions in this file
#define APPNAME @"Move Coach"

/**
 *  This constant describes the text used by the Voice Over screen reader to describe the + button for adding a weight, diet or physical activity entry.
 */
#define ADD_BUTTON_ACCESSIBILITY_TEXT @"Add Button: Double Tap this button to add a weight, diet or physical activity entry"

#define SECONDS_PER_DAY 60*60*24

#define FIRST_REASON_JOIN_MOVE @"Tap here to enter your first reason for joining MOVE!"
#define SECOND_REASON_JOIN_MOVE @"Tap here to enter your second reason for joining MOVE!"

#define FIRST_CHANGE_IMPACT @"Tap here to enter the first thing this change will impact";
#define SECOND_CHANGE_IMPACT @"Tap here to enter the second thing this change will impact";

#define FIRST_PASMART_GOAL_PROMPT @"Tap here to enter your first Physical Activity SMART goal.";
#define SECOND_PASMART_GOAL_PROMPT @"Tap here to enter your second Physical Activity SMART goal.";
#define FIRST_DIETSMART_GOAL_PROMPT @"Tap here to enter your first Dietary SMART goal.";
#define SECOND_DIETSMART_GOAL_PROMPT @"Tap here to enter your second Dietary SMART goal.";
#define SMART_GOAL_DATE_PROMPT_NEW @"Tap here to enter a check-in date for these goals.";
#define SMART_GOAL_DATE_PROMPT_UPDATE @"Tap here to update the check-in date for these goals.";
#define ACCESSIBLE_SMART_GOAL_DATE_PROMPT_NEW @"Double tap to enter a check-in date for these goals.";
#define ACCESSIBLE_SMART_GOAL_DATE_PROMPT_UPDATE @"Double tap to update the check-in date for these goals.";

#define kWEIGHT_GOAL_EVENT_ID @"weightGoalEvent"
#define kPHYSICALACTIVITY_GOAL_EVENT_ID @"physicalActivityGoalEvent"
#define kDIET_GOAL_EVENT_ID @"dietGoalEvent"

/**
 *  This Class provides Utility methods to facilitate code reuse and constant definitions used by various Classes in the app.  It follows the Singleton design pattern so only one instance is ever created.
 *
 */
@interface Utilities : NSObject 

/**
 *  This Class follows the Singleton design pattern so only one instance is ever created.  Calling Classes access the singleton with this method.
 *
 *  @return Utilities instance used to access all Utilities fields and methods
 */
+ (Utilities *) defaultUtilities;

/**
 *  This method provides reusable code to set the disclosure indicator on a UITableViewCell.  This is called from each Guide#ViewController class and is used to display the correct disclosure indicator for each chapter within each guide.  If a chapter has been viewed, it will have a checkmark.  If a chapter has not been viewed, it will have a disclosure arrow.  If the chapter is the last chapter visited, it will be highlighted in blue.  
 *
 *  @param tableView - The specific UITableView that is managing the layout of this table
 *
 *  @param cell - The UITableViewCell that needs to display a disclosure indicator
 *
 *  @param indexPath - The indexPath of the cell in question.  This gives us the table Section and Row where the cell is located.
 *
 *  @param guideNumber - The specific guide that is being displayed.
 */
- (void) setCellAccessory:(UITableView *)tableView withCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath forGuide:(int)guideNumber;


//Computational methods - arithmetic & date/time
/**
 *  This utility method generates a pseudorandom number within a range
 *
 *  @param min the minimum range for random number generation
 *
 *  @param max the maximum range for random number generation
 *
 *  @return a pseudorandom float between min and max
 */
- (float)floatInRangeMin: (int)min andMax: (int)max;

/**
 *  This utility method determines if two dates are the same day, disregarding time
 *
 *  @param  date0 the first date
 *
 *  @param  date1 the second date
 *
 *  @return YES if the date0 and date1 represent the same DAY, NO if they do not
 */
- (BOOL)isSameDay: (NSDate*)date0 as: (NSDate*)date1;

/**
 *  Reusable static instance of NSDateFormatter.
 *
 *  Use this to minimize overhead throughout MCM.
 */
- (NSDateFormatter *)getDateFormatter;

/**
 *  This method creates a custom title view for any navigation item that needs it.  This allows more control over the accessibility elements of the navigation item.
 *
 *  @param  title - the title as it will appear on the navigation bar
 *
 *  @param label - The accessibilty label that will be read by voice over
 *
 *  @return The UILabel that can be set to self.navigationItem.titleView  
 */
- (UILabel *) createCustomHeaderForNavigationItem:(NSString *)title withAccessibilityLabel:(NSString *)label;

/**
 *  This method calculates the Body Mass Index (BMI) given a weight and height
 *
 *  @param weight - the weight from which BMI is to be calculated
 *
 *  @param height - the height from which BMI is to be calculated
 */
- (float) calculateBMIGivenWeight:(float)weight andHeight:(float)height;


//Chapter level TableView helper methods
/*
 *  This method is currently unused but kept for future functionality.  This is used when forcing the user to navigate through the chapters within a guide a specific order
 
- (NSString *) getNextValidIndexPathKey:(NSIndexPath *)indexPath withTableView:(UITableView *)tableView forGuide:(int)guideNumber;
*/

/*
 * takes a date and normalizes it to be used as a key into a hash.  the time is set to a normalized value so all objects entered on this date can be sorted by this key.
 *
 *
 */
- (NSDate *) getDateConfiguredAsKey:(NSDate *)date;

/**
 *
 * takes a string that is all caps and returns it in 'sentence case'
 *
 */
- (NSString *) removeAllCapsFromString:(NSString *)allCapsString;

//to assist with debugging
- (void)listSubviewsOfView:(UIView *)view addBorder:(BOOL)border;

- (UIView *)createFooterViewWithDoneButtonForViewController:(UIViewController *)viewController;

- (int)daysBetween:(NSDate *)dt1 and:(NSDate *)dt2;

- (UIColor *)getColorForBMI:(float)bmi;

- (UIColor *) getBreakfastColor;
- (UIColor *) getLunchColor;
- (UIColor *) getDinnerColor;
- (UIColor *) getSnacksColor;

@end
