//
//  WeightGoal.h
//  MoveCoachMobile
//
//  Created by Joel Russell on 9/4/13.
//

#import <Foundation/Foundation.h>
#import "Utilities.h"
#import "Goal.h"

/**
 *  This Class is part of the Data Model and represents a single instance of a weight goal.  WeightGoal extends the Goal class, offering a number of context-specific, interrelated properties and methods that don't apply to either Activity or Diet Goals.
 */
@interface WeightGoal : Goal

/**
 *  This property represents the date associated with the weight goal (usually the date the goal was entered into the system)
 */
@property (nonatomic, strong)NSDate *date;

/**
 *  This is the baseline weight (usually starting weight) in pounds. Baseline weight may be used to infer both targetWeight and totalWeightLossGoal, assuming a default totalWeightLossGoalPercent of 5%.
 */
@property (nonatomic)float referenceWeight;

/**
 *  This is the user's current weight in pounds. Current weight may be used along with baseline weight and weight loss rate to infer the expectedGoalCompletionDate.
 */
@property (nonatomic)float currentWeight;

/**
 * This represents the user's selection for how much weight in pounds they will lose to achieve their weight loss goal.  This is tightly coupled with totalWeightLossGoalPercent and targetWeight
 */
@property (nonatomic)float totalWeightLossGoal;

/**
 * This represents the user's selection for how much weight as a percentage of their current weight they will lose to achieve their weight loss goal.  This is tightly coupled with totalWeightLossGoal and targetWeight
 */
@property (nonatomic)float totalWeightLossGoalPercent;

/**
 * This represents the user's selection for their target goal weight to achieve their weight loss goal.  This is tightly coupled with totalWeightLossGoal and totalWeightLossGoalPercent
 */
@property (nonatomic)float targetWeight;

/**
 * This represents the user's selection for how much weight in pounds they will lose per week.  Options are: 0 (maintain), 1/2, 1, 1 1/2, 2 pounds.  This is used to compute and display the Date the goal weight will be reached
 */
@property (nonatomic)float weightLossRate;

/**
 * This is the estimated completion date for the current weight goal, as inferred from the weightLossRate and the totalWeightLossGoal.
 */
//@property (nonatomic)NSDate *expectedGoalCompletionDate;

/**
 * This Boolean property is set to YES when any of the other properties of the WeightGoal is modified during a session.

@property (nonatomic,getter=isModified)BOOL modified;
 */

/**
 *  This is the primary method to instantiate a weight goal object with baseline weight and set date to the current date. The totalWeightLossGoalPercent will default to 5%, and the totalWeightLossGoal and targetWeight will be inferred. The weightLossRate will default to 0 (maintain).
 *
 *  @param aWeight - The baseline weight
 */
- (id)initWithReferenceWeight:(float)aRefWeight;

/**
 *  This method allows a weight goal object to be created with a specific target weight and date.
 *
 *  @param aRefWeight - Baseline weight
 *
 *  @param aCurrWeight - Current weight
 *
 *  @param aDate - a specific date to set date to.
 */
- (id)initWithReferenceWeight:(float)aRefWeight andCurrentWeight:(float)aCurrWeight andDate:(NSDate *)aDate;

    /**
     *  This method allows a weight goal object to be created with a specific target weight and date.
     *
     *  @param aRefWeight - Baseline weight
     *
     *  @param aCurrWeight - Current weight
     *
     *  @param aDate - a specific date to set date to.
     */
- (id)initWithCurrentWeight:(float)aCurrWeight andDate:(NSDate *)aDate andPercentToLose:(float)percentToLose andWeightLossRate:(float)weightLossRate;
    
    
/**
 * This initializer allows instantiation of a new WeightGoal based on a prior Weight Goal.
 *
 * @param aWeightGoal - prior Weight Goal
 */
- (id)initWithWeightGoal:(WeightGoal *)aWeightGoal;

@end
